package raft.karga.sound;

import raft.karga.core.*;
import raft.karga.core.Logger;
import raft.karga.world.KConfig;
import com.threed.jpct.*;

import java.util.*;
import java.io.*;
import javax.sound.sampled.*;

/**
 *
 * @author hakan eryargi (r a f t)
 */
public class AmbientSound {
    
    private final Logger logger = Logger.getLogger();
    
    private final SoundBank soundBank;
    private final ClipData clipData;
    private final String name;
    private boolean disposed = false;
    
    private static Mixer mixer;
    static {
        for (Mixer.Info mixerInfo : AudioSystem.getMixerInfo()) {
            if ("Java Sound Audio Engine".equals(mixerInfo.getName())) {
                mixer = AudioSystem.getMixer(mixerInfo);
                break;
            }
        }
        if (mixer == null)
            Logger.getLogger().warning("couldnt get Java Sound Audio Engine");
    }
    
    /** Creates a new instance of AmbientSound */
    AmbientSound(SoundBank soundBank, String name) throws Exception {
        this.name = name;
        this.soundBank = soundBank;
        this.clipData = new ClipData(soundBank, name);
    }
    
    public void playOnce() {
        play(SoundBank.PLAY_ONCE);
    }
    
    public void playOnceAndDispose() {
        play(SoundBank.PLAY_ONCE);
        clipData.addLineListener(new LineListener() {
            public void update(LineEvent event) {
                if (event.getType() == LineEvent.Type.STOP)
                    javax.swing.SwingUtilities.invokeLater(new Runnable() { public void run() {
                        dispose();
                    }});
            }
        });
    }
    
    public void playLoop() {
        play(SoundBank.PLAY_LOOP);
    }
    
    /** starts to play sound with calculated pan and gain values */
    private void play(final boolean loop) {
        if (KConfig.mute || clipData.isRunning())
            return;
        
        checkDisposed();
        float gain = SoundBank.overAllGain + clipData.soundData.initialGain;
        if ((clipData.gain != null) && (clipData.gain.getValue() != gain))
            clipData.gain.setValue(gain);
        
        /*
        if (loop)
            clipData.clip.loop(Clip.LOOP_CONTINUOUSLY);
        else clipData.clip.start();
         */
        
        if (loop)
            clipData.loop();
        else clipData.start();
        
        /*
        if (clipData.clip != null) {
            if (loop)
                clipData.clip.loop(Clip.LOOP_CONTINUOUSLY);
            else clipData.clip.start();
        } else {
            new Thread() {
                public void run() {
                    clipData.sourceDataLine.start();
                    do {
                        clipData.sourceDataLine.write(clipData.soundData.bytes, 0, clipData.soundData.bytes.length);
                        //System.out.println("wrote " + clipData.soundData.bytes.length + " bytes");
                        clipData.sourceDataLine.drain();
                        //System.out.println("drained");
                    } while (loop && clipData.sourceDataLine.isRunning());
                    // Stop
                    clipData.sourceDataLine.stop();
                    //System.out.println("stopepd");
                    clipData.sourceDataLine.close();
                    //System.out.println("closed");
                }
            }.start();
        }
         */
        
        
        logger.fine("started to play {0}, loop: {1}", name, loop);
    }
    
    public boolean isRunning() {
        return clipData.isRunning();
    }
    
    public void stop() {
        if (!clipData.isRunning())
            return;
        
        clipData.stop();
    }
    
    public void dispose() {
        disposed = true;
        stop();
        clipData.close();
        soundBank.ambientSounds.remove(this);
    }
    
    private void checkDisposed() {
        if (disposed)
            throw new IllegalStateException("disposed");
    }
    
    /** called when sound settings (volume) changed */
    void update() {
        float gain = SoundBank.overAllGain + clipData.soundData.initialGain;
        if ((clipData.gain != null) && (clipData.gain.getValue() != gain))
            clipData.gain.setValue(gain);
    }
    
}
