package raft.karga.sound;

import raft.karga.core.Logger;

import javax.sound.sampled.*;

/**
 *
 * @author hakan eryargi (r a f t)
 */
class ClipData {
    
    final String name;
    private final Clip clip;
    private final SourceDataLine sourceDataLine;
    final FloatControl gain;
    final FloatControl pan;
    final SoundBank.Sound soundData;
    
    private static Mixer mixer;
    static {
        for (Mixer.Info mixerInfo : AudioSystem.getMixerInfo()) {
            if ("Java Sound Audio Engine".equals(mixerInfo.getName())) {
                mixer = AudioSystem.getMixer(mixerInfo);
                break;
            }
        }
        if (mixer == null)
            Logger.getLogger().warning("couldnt get Java Sound Audio Engine");
    }
    
    ClipData(SoundBank soundBank, String name) throws Exception {
        this.name = name;
        soundData = soundBank.getSound(name);
        
        DataLine dataLine;
        if (soundData.useClip) {
            clip = (mixer != null) ? (Clip) mixer.getLine(soundData.lineInfo) : (Clip) AudioSystem.getLine(soundData.lineInfo);
            clip.open(soundData.format, soundData.bytes, 0, soundData.bytes.length);
            dataLine = clip;
            sourceDataLine = null;
        } else {
            sourceDataLine = (mixer != null) ? (SourceDataLine) mixer.getLine(soundData.lineInfo) : (SourceDataLine) AudioSystem.getLine(soundData.lineInfo);
            sourceDataLine.open(soundData.format);
            dataLine = sourceDataLine;
            clip = null;
        }
        
        gain = dataLine.isControlSupported(FloatControl.Type.MASTER_GAIN) ?
            (FloatControl) dataLine.getControl(FloatControl.Type.MASTER_GAIN) : null;
        pan = dataLine.isControlSupported(FloatControl.Type.PAN) ?
            (FloatControl) dataLine.getControl(FloatControl.Type.PAN) : null;
    }
    
    public boolean isRunning() {
        return (clip != null) ? clip.isRunning() : sourceDataLine.isRunning();
    }
    
    public void stop() {
        if (clip != null)
            clip.stop();
        else sourceDataLine.stop();
    }
    
    public void close() {
        if (clip != null)
            clip.close();
        else sourceDataLine.close();
    }
    
    public void addLineListener(LineListener l) {
        if (clip != null)
            clip.addLineListener(l);
        else sourceDataLine.addLineListener(l);
    }
    
    public void start() {
        if (clip != null) {
            clip.start();
        } else {
            new Thread() {
                public void run() {
                    sourceDataLine.start();
                    sourceDataLine.write(soundData.bytes, 0, soundData.bytes.length);
                    sourceDataLine.drain();
                    sourceDataLine.stop();
                    sourceDataLine.close();
                }
            }.start();
        }
    }
    
    public void loop() {
        if (clip != null) {
            clip.loop(Clip.LOOP_CONTINUOUSLY);
        } else {
            new Thread() {
                public void run() {
                    sourceDataLine.start();
                    do {
                        sourceDataLine.write(soundData.bytes, 0, soundData.bytes.length);
                        sourceDataLine.drain();
                    } while (sourceDataLine.isRunning());
                    sourceDataLine.stop();
                    sourceDataLine.close();
                }
            }.start();
        }
    }
}
